#!/usr/bin/env sh

# “Master, we're in a tight spot.”
#
# IwazuScript by wakfu[СК]
# QuickBMS and the BMS script by Luigi Auriemma
# Original guide by sal4, posted in #tutorial channel in killer7 mod discord by kindofblue28
# Shell Script based on looking at pinejinx script and RiiConnect24 patcher script
# Have fun!

###Variables
version="1.0"
useragent="IwazuScript Killer7 Mod Script Linux v${version}"
urlquickbms="https://aluigi.altervista.org/papers/quickbms_linux.zip"
urlbmsscript="https://aluigi.altervista.org/bms/killer7_vfs.bms"
extractutil=""
tempdir="$(mktemp -d -t 'IwazuScript.XXXXXXXXXX')"
k7dir="${PWD}"
progress=(0 0 0 0 0)
COLUMNS=$(tput cols)


###Functions
##K7Setup
#Downloading QuickBMS
downloadqbms() {
	cd "$tempdir"
	if curl -f -L -k -s -S -o "${tempdir}/quickbms_linux.zip" -A "$useragent" "$urlquickbms" && extractutil && chmod +x "${tempdir}/quickbms"; then
		progress[0]=1
		refresh
	else
		printerr "QuickBMS download failed!"
		rm -r "$tempdir"
		exit 1
	fi
}

#Downloading BMS Script
downloadbmsscrpt() {
	cd "$tempdir"
	progress[1]=1
	refresh
	if curl -f -L -k -s -S -o "${tempdir}/killer7_vfs.bms" -A "$useragent" "$urlbmsscript"; then
		progress[1]=2
		refresh
	else
		printerr "BMS Script download failed!"
		rm -r "$tempdir"
		exit 1
	fi
}

#Extracting game files
extractgame() {
	cd "$tempdir"
	progress[2]=1
	refresh
	if ./quickbms "${tempdir}/killer7_vfs.bms" "${k7dir}/data.vfs" "${k7dir}" >"${k7dir}/quickbmslog.txt" 2>&1; then
		progress[2]=2
		refresh
	else
		printerr "Extracting game files failed!"
		rm -r "$tempdir"
		exit 1
	fi
}

#Cleanup
cleanup() {
	progress[3]=1
	refresh
	if mv -f "${k7dir}/data.vfs" "${k7dir}/data_og.vfs" && rm -r "$tempdir"; then
		progress[3]=2
		refresh
	else
		printerr "Cleaning up failed!"
		rm -r "$tempdir"
		exit 1
	fi
}

#Revert, aka Deleting extracted folders and renaming vfs back
revertback() {
	rm -r "${k7dir}/3DData"
	rm -r "${k7dir}/CharaGeki"
	rm -r "${k7dir}/fonts"
	rm -r "${k7dir}/hato"
	rm -r "${k7dir}/logo"
	rm -r "${k7dir}/return"
	rm -r "${k7dir}/scenario"
	rm -r "${k7dir}/sub"
	rm -r "${k7dir}/subtitle"
	rm -r "${k7dir}/Texture"
	rm -r "${k7dir}/tutorial"
	rm -r "${k7dir}/Wat"
	rm -r "${k7dir}/Yam"
	rm -r "${k7dir}/Zan"
	#Finally, rename
	mv -f "${k7dir}/data_og.vfs" "${k7dir}/data.vfs"
	progress[0]=1
	refreshrevert
}

##Screens
#Fluff
fluff() {
	clear
	cash
	printf "\n"
	print "      IwazuScript ${version}"
	print "        by wakfu[СК]"
	printf "\n"
	cash
	print "QuickBMS and the BMS script by Luigi Auriemma"
	print "Original guide by sal4"
	print "Have fun!"
	cash
	printf "\n"
}

#Main Screen
mainscreen() {
	fluff
	print "Press any key to start setting up Killer7"
}

#Refresh Screen (updates every phase completed)
refresh() {
	fluff
	if [ ${progress[0]} = 1 ]; then
		print "[X] Downloading QuickBMS"
	else
		print "[.] Downloading QuickBMS"
	fi
	printf "\n"

	if [ ${progress[1]} = 2 ]; then
		print "[X] Downloading BMS Script"
	elif [ ${progress[1]} = 1 ]; then
		print "[.] Downloading BMS Script"
	else
		print "[ ] Downloading BMS Script"
	fi
	printf "\n"

	if [ ${progress[2]} = 2 ]; then
		print "[X] Extracting game files"
	elif [ ${progress[2]} = 1 ]; then
		print "[.] Extracting game files"
	else
		print "[ ] Extracting game files"
	fi
	printf "\n"

	if [ ${progress[3]} = 2 ]; then
		print "[X] Cleaning up"
	elif [ ${progress[3]} = 1 ]; then
		print "[.] Cleaning up"
	else
		print "[ ] Cleaning up"
	fi
	printf "\n"

	if [ ${progress[4]} = 1 ]; then
		print "[X] Done!"
	else
		print "[ ] Done!"
	fi
	printf "\n"
}

#Revert Screen
revertscreen() {
	fluff
	print "It looks like you already setup your game!"
	printf "\n"
	print "Would you like to revert the game back to it's original state?"
	printf "\n"
	print "Press any key to continue"
	print "\033[0;91m!WARNING!\033[0m All of your mods will be deleted!"
}

#Revert Refresh Screen
refreshrevert() {
	fluff
	if [ ${progress[0]} = 1 ]; then
		print "[X] Reverting the game back"
	else
		print "[.] Reverting the game back"
	fi
	printf "\n"
	if [ ${progress[1]} = 1 ]; then
		print "[X] Done!"
	else
		print "[ ] Done!"
	fi
	printf "\n"
}

##Actions
#Print with word wrap
print() {
	printf "  ${1}\n" | fold -s -w $COLUMNS
}

#Print with word wrap (error)
printerr() {
	printf "\033[1;31mERROR:\033[0m ${1}\n" | fold -s -w $COLUMNS
}

#Pause
pause() {
	read -rsn1
}

#Extract Utility
extractutil() {
	if command -v bsdtar >/dev/null 2>&1; then
		bsdtar -zxf "${tempdir}/quickbms_linux.zip" "quickbms"
	else
		unzip -q "${tempdir}/quickbms_linux.zip" "quickbms"
	fi
}

#Horizontal Cash Money
cash() {
	printf "\033[0;33m$"
	i=3
	while [ $i -le $COLUMNS ]
	do i=$((i+1))
		printf "${1:-\\033[0;32m=}"
	done
	printf "\033[0;33m$\033[0m\n"
}


###Start
##Checks
#Checking for renamed data.vfs, if it exists we reverting
if [ -f "./data_og.vfs" ]; then
	progress=(0 0)
	rm -r "$tempdir"
	revertscreen
	pause
	refreshrevert
	revertback
	#We're done
	progress[1]=1
	refreshrevert
	exit
fi

#Checking Prerequisites
if ! command -v curl >/dev/null 2>&1; then
	printerr "It appears that you don't have curl installed! IwazuScript can't run!"
	rm -r "$tempdir"
	exit 1
fi

#Checking extract util
if ! command -v bsdtar >/dev/null 2>&1 && ! command -v unzip >/dev/null 2>&1; then
	printerr "Haven't found a compatible unarchiver! Please install bsdtar (libarchive) or unzip."
	rm -r "$tempdir"
	exit 1
fi

#Checking internet connection
if ! curl -Is https://aluigi.altervista.org | grep -q 200; then
	printerr "Can't reach aluigi.altervista.org! Are you sure that you've got internet? Is IwazuScript allowed to access it in your firewall settings? Is the website reachable in your web browser?"
	rm -r "$tempdir"
	exit 1
fi

#Checking for killer7 files, if not found file check failed
if ! [ -f "./Killer7Win.exe" ] || ! [ -f "./data.vfs" ]; then
	printerr "File check failed! Are you sure the script is in Killer7's root directory? Check that the data.vfs file is in the same folder as the script."
	rm -r "$tempdir"
	exit 1
fi

##Main
#Trap deleting tempdir
trap "rm -r \"${tempdir}\"; printf \"\n\"; printf \"Exiting...\n\"; exit" INT TERM
mainscreen
pause
refresh
downloadqbms
downloadbmsscrpt
extractgame
cleanup
#We're done
progress[4]=1
refresh
exit
